@@ -25,6 +25,29 @@ class Proxy(proxy.Proxy):
 
     retriable_status_codes = _common.RETRIABLE_STATUS_CODES
 
+    def _get_with_fields(self, resource_type, value, fields=None):
+        """Fetch a bare metal resource.
+
+        :param resource_type: The type of resource to get.
+        :type resource_type: :class:`~openstack.resource.Resource`
+        :param value: The value to get. Can be either the ID of a
+                      resource or a :class:`~openstack.resource.Resource`
+                      subclass.
+        :param fields: Limit the resource fields to fetch.
+
+        :returns: The result of the ``fetch``
+        :rtype: :class:`~openstack.resource.Resource`
+        """
+        res = self._get_resource(resource_type, value)
+        kwargs = {}
+        if fields:
+            kwargs['fields'] = _common.comma_separated_list(fields)
+        return res.fetch(
+            self,
+            error_message="No {resource_type} found for {value}".format(
+                resource_type=resource_type.__name__, value=value),
+            **kwargs)
+
     def chassis(self, details=False, **query):
         """Retrieve a generator of chassis.
 
@@ -85,17 +108,18 @@ class Proxy(proxy.Proxy):
         return self._find(_chassis.Chassis, name_or_id,
                           ignore_missing=ignore_missing)
 
-    def get_chassis(self, chassis):
+    def get_chassis(self, chassis, fields=None):
         """Get a specific chassis.
 
         :param chassis: The value can be the ID of a chassis or a
             :class:`~openstack.baremetal.v1.chassis.Chassis` instance.
+        :param fields: Limit the resource fields to fetch.
 
         :returns: One :class:`~openstack.baremetal.v1.chassis.Chassis`
         :raises: :class:`~openstack.exceptions.ResourceNotFound` when no
             chassis matching the name or ID could be found.
         """
-        return self._get(_chassis.Chassis, chassis)
+        return self._get_with_fields(_chassis.Chassis, chassis, fields=fields)
 
     def update_chassis(self, chassis, **attrs):
         """Update a chassis.
@@ -239,17 +263,18 @@ class Proxy(proxy.Proxy):
         return self._find(_node.Node, name_or_id,
                           ignore_missing=ignore_missing)
 
-    def get_node(self, node):
+    def get_node(self, node, fields=None):
         """Get a specific node.
 
         :param node: The value can be the name or ID of a node or a
             :class:`~openstack.baremetal.v1.node.Node` instance.
+        :param fields: Limit the resource fields to fetch.
 
         :returns: One :class:`~openstack.baremetal.v1.node.Node`
         :raises: :class:`~openstack.exceptions.ResourceNotFound` when no
             node matching the name or ID could be found.
         """
-        return self._get(_node.Node, node)
+        return self._get_with_fields(_node.Node, node, fields=fields)
 
     def update_node(self, node, retry_on_conflict=True, **attrs):
         """Update a node.
@@ -280,6 +305,10 @@ class Proxy(proxy.Proxy):
             being locked. However, when setting ``instance_id``, this is
             a normal code and should not be retried.
 
+        See `Update Node
+        <https://docs.openstack.org/api-ref/baremetal/?expanded=update-node-detail#update-node>`_
+        for details.
+
         :returns: The updated node.
         :rtype: :class:`~openstack.baremetal.v1.node.Node`
         """
@@ -322,6 +351,19 @@ class Proxy(proxy.Proxy):
                                        rescue_password=rescue_password,
                                        wait=wait, timeout=timeout)
 
+    def set_node_boot_device(self, node, boot_device, persistent=False):
+        """Set node boot device
+
+        :param node: The value can be the name or ID of a node or a
+            :class:`~openstack.baremetal.v1.node.Node` instance.
+        :param boot_device: Boot device to assign to the node.
+        :param persistent: If the boot device change is maintained after node
+            reboot
+        :return: The updated :class:`~openstack.baremetal.v1.node.Node`
+        """
+        res = self._get_resource(_node.Node, node)
+        return res.set_boot_device(self, boot_device, persistent=persistent)
+
     def wait_for_nodes_provision_state(self, nodes, expected_state,
                                        timeout=None,
                                        abort_on_failed_state=True):
@@ -340,6 +382,9 @@ class Proxy(proxy.Proxy):
 
         :return: The list of :class:`~openstack.baremetal.v1.node.Node`
             instances that reached the requested state.
+        :raises: :class:`~openstack.exceptions.ResourceFailure` if a node
+            reaches an error state and ``abort_on_failed_state`` is ``True``.
+        :raises: :class:`~openstack.exceptions.ResourceTimeout` on timeout.
         """
         log_nodes = ', '.join(n.id if isinstance(n, _node.Node) else n
                               for n in nodes)
@@ -532,23 +577,18 @@ class Proxy(proxy.Proxy):
         return self._find(_port.Port, name_or_id,
                           ignore_missing=ignore_missing)
 
-    def get_port(self, port, **query):
+    def get_port(self, port, fields=None):
         """Get a specific port.
 
         :param port: The value can be the ID of a port or a
             :class:`~openstack.baremetal.v1.port.Port` instance.
-        :param dict query: Optional query parameters to be sent to restrict
-            the port properties returned. Available parameters include:
-
-            * ``fields``: A list containing one or more fields to be returned
-              in the response. This may lead to some performance gain
-              because other fields of the resource are not refreshed.
+        :param fields: Limit the resource fields to fetch.
 
         :returns: One :class:`~openstack.baremetal.v1.port.Port`
         :raises: :class:`~openstack.exceptions.ResourceNotFound` when no
             port matching the name or ID could be found.
         """
-        return self._get(_port.Port, port, **query)
+        return self._get_with_fields(_port.Port, port, fields=fields)
 
     def update_port(self, port, **attrs):
         """Update a port.
@@ -655,23 +695,19 @@ class Proxy(proxy.Proxy):
         return self._find(_portgroup.PortGroup, name_or_id,
                           ignore_missing=ignore_missing)
 
-    def get_port_group(self, port_group, **query):
+    def get_port_group(self, port_group, fields=None):
         """Get a specific port group.
 
         :param port_group: The value can be the name or ID of a chassis or a
             :class:`~openstack.baremetal.v1.port_group.PortGroup` instance.
-        :param dict query: Optional query parameters to be sent to restrict
-            the port group properties returned. Available parameters include:
-
-            * ``fields``: A list containing one or more fields to be returned
-              in the response. This may lead to some performance gain
-              because other fields of the resource are not refreshed.
+        :param fields: Limit the resource fields to fetch.
 
         :returns: One :class:`~openstack.baremetal.v1.port_group.PortGroup`
         :raises: :class:`~openstack.exceptions.ResourceNotFound` when no
             port group matching the name or ID could be found.
         """
-        return self._get(_portgroup.PortGroup, port_group, **query)
+        return self._get_with_fields(_portgroup.PortGroup, port_group,
+                                     fields=fields)
 
     def update_port_group(self, port_group, **attrs):
         """Update a port group.
@@ -822,17 +858,45 @@ class Proxy(proxy.Proxy):
         """
         return self._create(_allocation.Allocation, **attrs)
 
-    def get_allocation(self, allocation):
+    def get_allocation(self, allocation, fields=None):
         """Get a specific allocation.
 
         :param allocation: The value can be the name or ID of an allocation or
             a :class:`~openstack.baremetal.v1.allocation.Allocation` instance.
+        :param fields: Limit the resource fields to fetch.
 
         :returns: One :class:`~openstack.baremetal.v1.allocation.Allocation`
         :raises: :class:`~openstack.exceptions.ResourceNotFound` when no
             allocation matching the name or ID could be found.
         """
-        return self._get(_allocation.Allocation, allocation)
+        return self._get_with_fields(_allocation.Allocation, allocation,
+                                     fields=fields)
+
+    def update_allocation(self, allocation, **attrs):
+        """Update an allocation.
+
+        :param allocation: The value can be the name or ID of an allocation or
+            a :class:`~openstack.baremetal.v1.allocation.Allocation` instance.
+        :param dict attrs: The attributes to update on the allocation
+            represented by the ``allocation`` parameter.
+
+        :returns: The updated allocation.
+        :rtype: :class:`~openstack.baremetal.v1.allocation.Allocation`
+        """
+        return self._update(_allocation.Allocation, allocation, **attrs)
+
+    def patch_allocation(self, allocation, patch):
+        """Apply a JSON patch to the allocation.
+
+        :param allocation: The value can be the name or ID of an allocation or
+            a :class:`~openstack.baremetal.v1.allocation.Allocation` instance.
+        :param patch: JSON patch to apply.
+
+        :returns: The updated allocation.
+        :rtype: :class:`~openstack.baremetal.v1.allocation.Allocation`
+        """
+        return self._get_resource(_allocation.Allocation,
+                                  allocation).patch(self, patch)
 
     def delete_allocation(self, allocation, ignore_missing=True):
         """Delete an allocation.
@@ -865,6 +929,50 @@ class Proxy(proxy.Proxy):
 
         :returns: The instance of the allocation.
         :rtype: :class:`~openstack.baremetal.v1.allocation.Allocation`.
+        :raises: :class:`~openstack.exceptions.ResourceFailure` if allocation
+            fails and ``ignore_error`` is ``False``.
+        :raises: :class:`~openstack.exceptions.ResourceTimeout` on timeout.
         """
         res = self._get_resource(_allocation.Allocation, allocation)
         return res.wait(self, timeout=timeout, ignore_error=ignore_error)
+
+    def add_node_trait(self, node, trait):
+        """Add a trait to a node.
+
+        :param node: The value can be the name or ID of a node or a
+            :class:`~openstack.baremetal.v1.node.Node` instance.
+        :param trait: trait to remove from the node.
+        :returns: The updated node
+        """
+        res = self._get_resource(_node.Node, node)
+        return res.add_trait(self, trait)
+
+    def remove_node_trait(self, node, trait, ignore_missing=True):
+        """Remove a trait from a node.
+
+        :param node: The value can be the name or ID of a node or a
+            :class:`~openstack.baremetal.v1.node.Node` instance.
+        :param trait: trait to remove from the node.
+        :param bool ignore_missing: When set to ``False``, an exception
+            :class:`~openstack.exceptions.ResourceNotFound` will be raised
+            when the trait could not be found. When set to ``True``, no
+            exception will be raised when attempting to delete a non-existent
+            trait.
+        :returns: The updated :class:`~openstack.baremetal.v1.node.Node`
+        """
+        res = self._get_resource(_node.Node, node)
+        return res.remove_trait(self, trait, ignore_missing=ignore_missing)
+
+    def set_node_traits(self, node, traits):
+        """Set traits for a node.
+
+        Removes any existing traits and adds the traits passed in to this
+        method.
+
+        :param node: The value can be the name or ID of a node or a
+            :class:`~openstack.baremetal.v1.node.Node` instance.
+        :param traits: list of traits to add to the node.
+        :returns: The updated :class:`~openstack.baremetal.v1.node.Node`
+        """
+        res = self._get_resource(_node.Node, node)
+        return res.set_traits(self, traits)
