@@ -340,7 +340,9 @@ def _get_interface_ip(cloud, server):
 def get_groups_from_server(cloud, server, server_vars):
     groups = []
 
-    region = cloud.config.region_name
+    # NOTE(efried): This is hardcoded to 'compute' because this method is only
+    # used from ComputeCloudMixin.
+    region = cloud.config.get_region_name('compute')
     cloud_name = cloud.name
 
     # Create a group for the cloud
