@@ -33,6 +33,7 @@ from openstack.cloud import _object_store
 from openstack.cloud import meta
 from openstack.cloud import _utils
 import openstack.config
+from openstack.config import cloud_region as cloud_region_mod
 from openstack import proxy
 
 DEFAULT_SERVER_AGE = 5
@@ -226,27 +227,16 @@ class _OpenStackCloudMixin(object):
         for key, value in kwargs.items():
             params['auth'][key] = value
 
-        # TODO(mordred) Replace this chunk with the next patch that allows
-        # passing a Session to CloudRegion.
-        # Closure to pass to OpenStackConfig to ensure the new cloud shares
-        # the Session with the current cloud. This will ensure that version
-        # discovery cache will be re-used.
-        def session_constructor(*args, **kwargs):
-            # We need to pass our current keystone session to the Session
-            # Constructor, otherwise the new auth plugin doesn't get used.
-            return keystoneauth1.session.Session(
-                session=self.session,
-                discovery_cache=self.config._discovery_cache)
-
-        cloud_config = config.get_one(
-            session_constructor=session_constructor,
-            **params)
+        cloud_region = config.get_one(**params)
+        # Attach the discovery cache from the old session so we won't
+        # double discover.
+        cloud_region._discovery_cache = self.session._discovery_cache
         # Override the cloud name so that logging/location work right
-        cloud_config._name = self.name
-        cloud_config.config['profile'] = self.name
+        cloud_region._name = self.name
+        cloud_region.config['profile'] = self.name
         # Use self.__class__ so that we return whatever this if, like if it's
         # a subclass in the case of shade wrapping sdk.
-        return self.__class__(config=cloud_config)
+        return self.__class__(config=cloud_region)
 
     def connect_as_project(self, project):
         """Make a new OpenStackCloud object with a new project.
@@ -278,6 +268,53 @@ class _OpenStackCloudMixin(object):
             auth['project_name'] = project
         return self.connect_as(**auth)
 
+    def global_request(self, global_request_id):
+        """Make a new Connection object with a global request id set.
+
+        Take the existing settings from the current Connection and construct a
+        new Connection object with the global_request_id overridden.
+
+        .. code-block:: python
+
+          from oslo_context import context
+          cloud = openstack.connect(cloud='example')
+          # Work normally
+          servers = cloud.list_servers()
+          cloud2 = cloud.global_request(context.generate_request_id())
+          # cloud2 sends all requests with global_request_id set
+          servers = cloud2.list_servers()
+
+        Additionally, this can be used as a context manager:
+
+        .. code-block:: python
+
+          from oslo_context import context
+          c = openstack.connect(cloud='example')
+          # Work normally
+          servers = c.list_servers()
+          with c.global_request(context.generate_request_id()) as c2:
+              # c2 sends all requests with global_request_id set
+              servers = c2.list_servers()
+
+        :param global_request_id: The `global_request_id` to send.
+        """
+        params = copy.deepcopy(self.config.config)
+        cloud_region = cloud_region_mod.from_session(
+            session=self.session,
+            app_name=self.config._app_name,
+            app_version=self.config._app_version,
+            discovery_cache=self.session._discovery_cache,
+            **params)
+
+        # Override the cloud name so that logging/location work right
+        cloud_region._name = self.name
+        cloud_region.config['profile'] = self.name
+        # Use self.__class__ so that we return whatever this is, like if it's
+        # a subclass in the case of shade wrapping sdk.
+        new_conn = self.__class__(config=cloud_region)
+        new_conn.set_global_request_id(global_request_id)
+        return new_conn
+
     def _make_cache(self, cache_class, expiration_time, arguments):
         return dogpile.cache.make_region(
             function_key_generator=self._make_cache_key
@@ -358,7 +395,7 @@ class _OpenStackCloudMixin(object):
                 service_name=self.config.get_service_name(service_type),
                 interface=self.config.get_interface(service_type),
                 endpoint_override=self.config.get_endpoint(service_type),
-                region_name=self.config.region_name,
+                region_name=self.config.get_region_name(service_type),
                 statsd_prefix=self.config.get_statsd_prefix(),
                 statsd_client=self.config.get_statsd_client(),
                 prometheus_counter=self.config.get_prometheus_counter(),
@@ -374,7 +411,7 @@ class _OpenStackCloudMixin(object):
             service_name=self.config.get_service_name(service_type),
             interface=self.config.get_interface(service_type),
             endpoint_override=self.config.get_endpoint(service_type),
-            region_name=self.config.region_name,
+            region_name=self.config.get_region_name(service_type),
             min_version=min_version,
             max_version=max_version)
 
@@ -413,10 +450,11 @@ class _OpenStackCloudMixin(object):
             interface=self.config.get_interface(service_type),
             endpoint_override=self.config.get_endpoint(
                 service_type) or endpoint_override,
-            region_name=self.config.region_name)
+            region_name=self.config.get_region_name(service_type))
 
     def _is_client_version(self, client, version):
-        client_name = '_{client}_client'.format(client=client)
+        client_name = '_{client}_client'.format(
+            client=client.replace('-', '_'))
         client = getattr(self, client_name)
         return client._version_matches(version)
 
@@ -462,9 +500,32 @@ class _OpenStackCloudMixin(object):
     def service_catalog(self):
         return self._keystone_catalog.catalog
 
-    def endpoint_for(self, service_type, interface='public'):
-        return self._keystone_catalog.url_for(
-            service_type=service_type, interface=interface)
+    def endpoint_for(self, service_type, interface=None, region_name=None):
+        """Return the endpoint for a given service.
+
+        Respects config values for Connection, including
+        ``*_endpoint_override``. For direct values from the catalog
+        regardless of overrides, see
+        :meth:`~openstack.config.cloud_region.CloudRegion.get_endpoint_from_catalog`
+
+        :param service_type: Service Type of the endpoint to search for.
+        :param interface:
+            Interface of the endpoint to search for. Optional, defaults to
+            the configured value for interface for this Connection.
+        :param region_name:
+            Region Name of the endpoint to search for. Optional, defaults to
+            the configured value for region_name for this Connection.
+
+        :returns: The endpoint of the service, or None if not found.
+        """
+
+        endpoint_override = self.config.get_endpoint(service_type)
+        if endpoint_override:
+            return endpoint_override
+        return self.config.get_endpoint_from_catalog(
+            service_type=service_type,
+            interface=interface,
+            region_name=region_name)
 
     @property
     def auth_token(self):
@@ -529,7 +590,9 @@ class _OpenStackCloudMixin(object):
     def _get_current_location(self, project_id=None, zone=None):
         return munch.Munch(
             cloud=self.name,
-            region_name=self.config.region_name,
+            # TODO(efried): This is wrong, but it only seems to be used in a
+            # repr; can we get rid of it?
+            region_name=self.config.get_region_name(),
             zone=zone,
             project=self._get_project_info(project_id),
         )
@@ -648,9 +711,6 @@ class _OpenStackCloudMixin(object):
     def get_name(self):
         return self.name
 
-    def get_region(self):
-        return self.config.region_name
-
     def get_session_endpoint(self, service_key):
         try:
             return self.config.get_session_endpoint(service_key)
@@ -666,12 +726,12 @@ class _OpenStackCloudMixin(object):
                 " {error}".format(
                     service=service_key,
                     cloud=self.name,
-                    region=self.config.region_name,
+                    region=self.config.get_region_name(service_key),
                     error=str(e)))
         return endpoint
 
     def has_service(self, service_key):
-        if not self.config.config.get('has_%s' % service_key, True):
+        if not self.config.has_service(service_key):
             # TODO(mordred) add a stamp here so that we only report this once
             if not (service_key in self._disable_warnings
                     and self._disable_warnings[service_key]):
