@@ -9,6 +9,7 @@
 # WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 # License for the specific language governing permissions and limitations
 # under the License.
+import warnings
 
 from openstack.compute.v2 import aggregate as _aggregate
 from openstack.compute.v2 import availability_zone
@@ -25,6 +26,7 @@ from openstack.compute.v2 import server_interface as _server_interface
 from openstack.compute.v2 import server_ip
 from openstack.compute.v2 import service as _service
 from openstack.compute.v2 import volume_attachment as _volume_attachment
+from openstack.network.v2 import security_group as _sg
 from openstack import proxy
 from openstack import resource
 
@@ -122,15 +124,16 @@ class Proxy(proxy.Proxy):
         flv = _flavor.FlavorDetail if details else _flavor.Flavor
         return self._list(flv, **query)
 
-    def aggregates(self):
+    def aggregates(self, **query):
         """Return a generator of aggregate
 
+        :param kwargs query: Optional query parameters to be sent to limit
+            the aggregates being returned.
+
         :returns: A generator of aggregate
         :rtype: class: `~openstack.compute.v2.aggregate.Aggregate`
         """
-        aggregate = _aggregate.Aggregate
-
-        return self._list(aggregate)
+        return self._list(_aggregate.Aggregate, **query)
 
     def get_aggregate(self, aggregate):
         """Get a single host aggregate
@@ -274,16 +277,18 @@ class Proxy(proxy.Proxy):
         """Return a generator of images
 
         :param bool details: When ``True``, returns
-            :class:`~openstack.compute.v2.image.ImageDetail` objects,
-            otherwise :class:`~openstack.compute.v2.image.Image`.
+            :class:`~openstack.compute.v2.image.Image` objects with all
+            available properties, otherwise only basic properties are returned.
             *Default: ``True``*
         :param kwargs query: Optional query parameters to be sent to limit
                                  the resources being returned.
 
         :returns: A generator of image objects
         """
-        img = _image.ImageDetail if details else _image.Image
-        return self._list(img, **query)
+        warnings.warn('This API is deprecated and may disappear shortly',
+                      DeprecationWarning)
+        base_path = '/images/detail' if details else None
+        return self._list(_image.Image, base_path=base_path, **query)
 
     def _get_base_resource(self, res, base):
         # Metadata calls for Image and Server can work for both those
@@ -298,9 +303,7 @@ class Proxy(proxy.Proxy):
         """Return a dictionary of metadata for an image
 
         :param image: Either the ID of an image or a
-                       :class:`~openstack.compute.v2.image.Image` or
-                       :class:`~openstack.compute.v2.image.ImageDetail`
-                       instance.
+            :class:`~openstack.compute.v2.image.Image` instance.
 
         :returns: A :class:`~openstack.compute.v2.image.Image` with only the
                   image's metadata. All keys and values are Unicode text.
@@ -315,9 +318,7 @@ class Proxy(proxy.Proxy):
         """Update metadata for an image
 
         :param image: Either the ID of an image or a
-                       :class:`~openstack.compute.v2.image.Image` or
-                       :class:`~openstack.compute.v2.image.ImageDetail`
-                       instance.
+            :class:`~openstack.compute.v2.image.Image` instance.
         :param kwargs metadata: Key/value pairs to be updated in the image's
                                 metadata. No other metadata is modified
                                 by this call. All keys and values are stored
@@ -338,9 +339,7 @@ class Proxy(proxy.Proxy):
         Note: This method will do a HTTP DELETE request for every key in keys.
 
         :param image: Either the ID of an image or a
-                       :class:`~openstack.compute.v2.image.Image` or
-                       :class:`~openstack.compute.v2.image.ImageDetail`
-                       instance.
+            :class:`~openstack.compute.v2.image.Image` instance.
         :param keys: The keys to delete.
 
         :rtype: ``None``
@@ -403,13 +402,15 @@ class Proxy(proxy.Proxy):
         return self._find(_keypair.Keypair, name_or_id,
                           ignore_missing=ignore_missing)
 
-    def keypairs(self):
+    def keypairs(self, **query):
         """Return a generator of keypairs
 
+        :param kwargs query: Optional query parameters to be sent to limit
+            the resources being returned.
         :returns: A generator of keypair objects
         :rtype: :class:`~openstack.compute.v2.keypair.Keypair`
         """
-        return self._list(_keypair.Keypair)
+        return self._list(_keypair.Keypair, **query)
 
     def get_limits(self):
         """Retrieve limits that are applied to the project's account
@@ -484,45 +485,18 @@ class Proxy(proxy.Proxy):
         """Retrieve a generator of servers
 
         :param bool details: When set to ``False``
-                    :class:`~openstack.compute.v2.server.Server` instances
-                    will be returned. The default, ``True``, will cause
-                    :class:`~openstack.compute.v2.server.ServerDetail`
-                    instances to be returned.
+            instances with only basic data will be returned. The default,
+            ``True``, will cause instances with full data to be returned.
         :param kwargs query: Optional query parameters to be sent to limit
-            the servers being returned.  Available parameters include:
-
-            * changes_since: A time/date stamp for when the server last changed
-                             status.
-            * image: An image resource or ID.
-            * flavor: A flavor resource or ID.
-            * name: Name of the server as a string.  Can be queried with
-                    regular expressions. The regular expression
-                    ?name=bob returns both bob and bobb.  If you must match on
-                    only bob, you can use a regular expression that
-                    matches the syntax of the underlying database server that
-                    is implemented for Compute, such as MySQL or PostgreSQL.
-            * status: Value of the status of the server so that you can filter
-                    on "ACTIVE" for example.
-            * host: Name of the host as a string.
-            * all_projects: Flag to request servers be returned from all
-                            projects, not just the currently scoped one.
-            * limit: Requests a specified page size of returned items from the
-                     query.  Returns a number of items up to the specified
-                     limit value. Use the limit parameter to make an initial
-                     limited request and use the ID of the last-seen item from
-                     the response as the marker parameter value in a subsequent
-                     limited request.
-            * marker: Specifies the ID of the last-seen item. Use the limit
-                      parameter to make an initial limited request and use the
-                      ID of the last-seen item from the response as the marker
-                      parameter value in a subsequent limited request.
+            the servers being returned. Available parameters can be seen
+            under https://docs.openstack.org/api-ref/compute/#list-servers
 
         :returns: A generator of server instances.
         """
         if all_projects:
             query['all_projects'] = True
-        srv = _server.ServerDetail if details else _server.Server
-        return self._list(srv, **query)
+        base_path = '/servers/detail' if details else None
+        return self._list(_server.Server, base_path=base_path, **query)
 
     def update_server(self, server, **attrs):
         """Update a server
@@ -650,39 +624,58 @@ class Proxy(proxy.Proxy):
         server = self._get_resource(_server.Server, server)
         server.revert_resize(self)
 
-    def create_server_image(self, server, name, metadata=None):
+    def create_server_image(self, server, name, metadata=None, wait=False,
+                            timeout=120):
         """Create an image from a server
 
         :param server: Either the ID of a server or a
-                       :class:`~openstack.compute.v2.server.Server` instance.
+            :class:`~openstack.compute.v2.server.Server` instance.
         :param str name: The name of the image to be created.
         :param dict metadata: A dictionary of metadata to be set on the image.
 
-        :returns: None
+        :returns: :class:`~openstack.image.v2.image.Image` object.
         """
         server = self._get_resource(_server.Server, server)
-        server.create_image(self, name, metadata)
+        image_id = server.create_image(self, name, metadata)
+
+        self._connection.list_images.invalidate(self)
+        image = self._connection.get_image(image_id)
+
+        if not wait:
+            return image
+        return self._connection.wait_for_image(image, timeout=timeout)
+
+    def fetch_server_security_groups(self, server):
+        """Fetch security groups with details for a server.
+
+        :param server: Either the ID of a server or a
+            :class:`~openstack.compute.v2.server.Server` instance.
+
+        :returns: updated :class:`~openstack.compute.v2.server.Server` instance
+        """
+        server = self._get_resource(_server.Server, server)
+        return server.fetch_security_groups(self)
 
     def add_security_group_to_server(self, server, security_group):
         """Add a security group to a server
 
         :param server: Either the ID of a server or a
-                       :class:`~openstack.compute.v2.server.Server` instance.
-        :param security_group: Either the ID of a security group or a
+            :class:`~openstack.compute.v2.server.Server` instance.
+        :param security_group: Either the ID, Name of a security group or a
             :class:`~openstack.network.v2.security_group.SecurityGroup`
             instance.
 
         :returns: None
         """
         server = self._get_resource(_server.Server, server)
-        security_group_id = resource.Resource._get_id(security_group)
-        server.add_security_group(self, security_group_id)
+        security_group = self._get_resource(_sg.SecurityGroup, security_group)
+        server.add_security_group(self, security_group.name)
 
     def remove_security_group_from_server(self, server, security_group):
         """Remove a security group from a server
 
         :param server: Either the ID of a server or a
-                       :class:`~openstack.compute.v2.server.Server` instance.
+            :class:`~openstack.compute.v2.server.Server` instance.
         :param security_group: Either the ID of a security group or a
             :class:`~openstack.network.v2.security_group.SecurityGroup`
             instance.
@@ -690,8 +683,8 @@ class Proxy(proxy.Proxy):
         :returns: None
         """
         server = self._get_resource(_server.Server, server)
-        security_group_id = resource.Resource._get_id(security_group)
-        server.remove_security_group(self, security_group_id)
+        security_group = self._get_resource(_sg.SecurityGroup, security_group)
+        server.remove_security_group(self, security_group.name)
 
     def add_fixed_ip_to_server(self, server, network_id):
         """Adds a fixed IP address to a server instance.
@@ -932,16 +925,20 @@ class Proxy(proxy.Proxy):
                         interval=2, wait=120):
         """Wait for a server to be in a particular status.
 
-        :param res: The resource to wait on to reach the specified status.
-                    The resource must have a ``status`` attribute.
-        :type resource: A :class:`~openstack.resource.Resource` object.
+        :param server:
+            The :class:`~openstack.compute.v2.server.Server` to wait on
+            to reach the specified status.
+        :type server: :class:`~openstack.compute.v2.server.Server`:
         :param status: Desired status.
-        :param failures: Statuses that would be interpreted as failures.
+        :param failures:
+            Statuses that would be interpreted as failures.
         :type failures: :py:class:`list`
-        :param interval: Number of seconds to wait before to consecutive
-                         checks. Default to 2.
-        :param wait: Maximum number of seconds to wait before the change.
-                     Default to 120.
+        :param int interval:
+            Number of seconds to wait before to consecutive checks.
+            Default to 2.
+        :param int wait:
+            Maximum number of seconds to wait before the change.
+            Default to 120.
         :returns: The resource is returned on success.
         :raises: :class:`~openstack.exceptions.ResourceTimeout` if transition
                  to the desired status failed to occur in specified seconds.
@@ -1063,12 +1060,11 @@ class Proxy(proxy.Proxy):
         :rtype: :class:`~openstack.compute.v2.availability_zone.\
                         AvailabilityZone`
         """
-        if details:
-            az = availability_zone.AvailabilityZoneDetail
-        else:
-            az = availability_zone.AvailabilityZone
+        base_path = '/os-availability-zone/detail' if details else None
 
-        return self._list(az)
+        return self._list(
+            availability_zone.AvailabilityZone,
+            base_path=base_path)
 
     def get_server_metadata(self, server):
         """Return a dictionary of metadata for a server
@@ -1091,9 +1087,7 @@ class Proxy(proxy.Proxy):
         """Update metadata for a server
 
         :param server: Either the ID of a server or a
-                       :class:`~openstack.compute.v2.server.Server` or
-                       :class:`~openstack.compute.v2.server.ServerDetail`
-                       instance.
+            :class:`~openstack.compute.v2.server.Server` instance.
         :param kwargs metadata: Key/value pairs to be updated in the server's
                                 metadata. No other metadata is modified
                                 by this call. All keys and values are stored
@@ -1114,9 +1108,7 @@ class Proxy(proxy.Proxy):
         Note: This method will do a HTTP DELETE request for every key in keys.
 
         :param server: Either the ID of a server or a
-                       :class:`~openstack.compute.v2.server.Server` or
-                       :class:`~openstack.compute.v2.server.ServerDetail`
-                       instance.
+            :class:`~openstack.compute.v2.server.Server` instance.
         :param keys: The keys to delete
 
         :rtype: ``None``
@@ -1194,23 +1186,19 @@ class Proxy(proxy.Proxy):
         """
         return self._list(_server_group.ServerGroup, **query)
 
-    def hypervisors(self, details=False):
+    def hypervisors(self, details=False, **query):
         """Return a generator of hypervisor
 
         :param bool details: When set to the default, ``False``,
-                    :class:`~openstack.compute.v2.hypervisor.Hypervisor`
-                    instances will be returned. ``True`` will cause
-                    :class:`~openstack.compute.v2.hypervisor.HypervisorDetail`
-                    instances to be returned.
+            :class:`~openstack.compute.v2.hypervisor.Hypervisor`
+            instances will be returned with only basic information populated.
+        :param kwargs query: Optional query parameters to be sent to limit
+            the resources being returned.
         :returns: A generator of hypervisor
         :rtype: class: `~openstack.compute.v2.hypervisor.Hypervisor`
         """
-        if details:
-            hypervisor = _hypervisor.HypervisorDetail
-        else:
-            hypervisor = _hypervisor.Hypervisor
-
-        return self._list(hypervisor)
+        base_path = '/os-hypervisors/detail' if details else None
+        return self._list(_hypervisor.Hypervisor, base_path=base_path, **query)
 
     def find_hypervisor(self, name_or_id, ignore_missing=True):
         """Find a hypervisor from name or id to get the corresponding info
