@@ -29,6 +29,7 @@ from keystoneauth1 import loading
 import yaml
 
 from openstack import _log
+from openstack.config import _util
 from openstack.config import cloud_region
 from openstack.config import defaults
 from openstack.config import vendors
@@ -72,19 +73,6 @@ BOOL_KEYS = ('insecure', 'cache')
 FORMAT_EXCLUSIONS = frozenset(['password'])
 
 
-# NOTE(dtroyer): This turns out to be not the best idea so let's move
-#                overriding defaults to a kwarg to OpenStackConfig.__init__()
-#                Remove this sometime in June 2015 once OSC is comfortably
-#                changed-over and global-defaults is updated.
-def set_default(key, value):
-    warnings.warn(
-        "Use of set_default() is deprecated. Defaults should be set with the "
-        "`override_defaults` parameter of OpenStackConfig."
-    )
-    defaults.get_defaults()  # make sure the dict is initialized
-    defaults._defaults[key] = value
-
-
 def get_boolean(value):
     if value is None:
         return False
@@ -95,44 +83,6 @@ def get_boolean(value):
     return False
 
 
-def _get_os_environ(envvar_prefix=None):
-    ret = defaults.get_defaults()
-    if not envvar_prefix:
-        # This makes the or below be OS_ or OS_ which is a no-op
-        envvar_prefix = 'OS_'
-    environkeys = [k for k in os.environ.keys()
-                   if (k.startswith('OS_') or k.startswith(envvar_prefix))
-                   and not k.startswith('OS_TEST')  # infra CI var
-                   and not k.startswith('OS_STD')   # oslotest var
-                   and not k.startswith('OS_LOG')   # oslotest var
-                   ]
-    for k in environkeys:
-        newkey = k.split('_', 1)[-1].lower()
-        ret[newkey] = os.environ[k]
-    # If the only environ keys are selectors or behavior modification, don't
-    # return anything
-    selectors = set([
-        'OS_CLOUD', 'OS_REGION_NAME',
-        'OS_CLIENT_CONFIG_FILE', 'OS_CLIENT_SECURE_FILE', 'OS_CLOUD_NAME'])
-    if set(environkeys) - selectors:
-        return ret
-    return None
-
-
-def _merge_clouds(old_dict, new_dict):
-    """Like dict.update, except handling nested dicts."""
-    ret = old_dict.copy()
-    for (k, v) in new_dict.items():
-        if isinstance(v, dict):
-            if k in ret:
-                ret[k] = _merge_clouds(ret[k], v)
-            else:
-                ret[k] = v.copy()
-        else:
-            ret[k] = v
-    return ret
-
-
 def _auth_update(old_dict, new_dict_source):
     """Like dict.update, except handling the nested dict called auth."""
     new_dict = copy.deepcopy(new_dict_source)
@@ -179,12 +129,20 @@ def _fix_argv(argv):
 
 class OpenStackConfig(object):
 
+    # These two attribute are to allow os-client-config to plumb in its
+    # local versions for backwards compat.
+    # They should not be used by anyone else.
+    _cloud_region_class = cloud_region.CloudRegion
+    _defaults_module = defaults
+
     def __init__(self, config_files=None, vendor_files=None,
                  override_defaults=None, force_ipv4=None,
                  envvar_prefix=None, secure_files=None,
                  pw_func=None, session_constructor=None,
                  app_name=None, app_version=None,
-                 load_yaml_config=True, load_envvars=True):
+                 load_yaml_config=True, load_envvars=True,
+                 statsd_host=None, statsd_port=None,
+                 statsd_prefix=None):
         self.log = _log.setup_logging('openstack.config')
         self._session_constructor = session_constructor
         self._app_name = app_name
@@ -208,7 +166,7 @@ class OpenStackConfig(object):
         if secure_file_override:
             self._secure_files.insert(0, secure_file_override)
 
-        self.defaults = defaults.get_defaults()
+        self.defaults = self._defaults_module.get_defaults()
         if override_defaults:
             self.defaults.update(override_defaults)
 
@@ -216,7 +174,7 @@ class OpenStackConfig(object):
         self.config_filename, self.cloud_config = self._load_config_file()
         _, secure_config = self._load_secure_file()
         if secure_config:
-            self.cloud_config = _merge_clouds(
+            self.cloud_config = _util.merge_clouds(
                 self.cloud_config, secure_config)
 
         if not self.cloud_config:
@@ -224,6 +182,10 @@ class OpenStackConfig(object):
         if 'clouds' not in self.cloud_config:
             self.cloud_config['clouds'] = {}
 
+        # Save the other config
+        self.extra_config = copy.deepcopy(self.cloud_config)
+        self.extra_config.pop('clouds', None)
+
         # Grab ipv6 preference settings from env
         client_config = self.cloud_config.get('client', {})
 
@@ -262,7 +224,7 @@ class OpenStackConfig(object):
         self.default_cloud = self._get_envvar('OS_CLOUD')
 
         if load_envvars:
-            envvars = _get_os_environ(envvar_prefix=envvar_prefix)
+            envvars = self._get_os_environ(envvar_prefix=envvar_prefix)
             if envvars:
                 self.cloud_config['clouds'][self.envvar_key] = envvars
                 if not self.default_cloud:
@@ -291,9 +253,9 @@ class OpenStackConfig(object):
         self._cache_path = CACHE_PATH
         self._cache_class = 'dogpile.cache.null'
         self._cache_arguments = {}
-        self._cache_expiration = {}
+        self._cache_expirations = {}
         if 'cache' in self.cloud_config:
-            cache_settings = self._normalize_keys(self.cloud_config['cache'])
+            cache_settings = _util.normalize_keys(self.cloud_config['cache'])
 
             # expiration_time used to be 'max_age' but the dogpile setting
             # is expiration_time. Support max_age for backwards compat.
@@ -313,8 +275,23 @@ class OpenStackConfig(object):
                 cache_settings.get('path', self._cache_path))
             self._cache_arguments = cache_settings.get(
                 'arguments', self._cache_arguments)
-            self._cache_expiration = cache_settings.get(
-                'expiration', self._cache_expiration)
+            self._cache_expirations = cache_settings.get(
+                'expiration', self._cache_expirations)
+
+        if load_yaml_config:
+            statsd_config = self.cloud_config.get('statsd', {})
+            statsd_host = statsd_host or statsd_config.get('host')
+            statsd_port = statsd_port or statsd_config.get('port')
+            statsd_prefix = statsd_prefix or statsd_config.get('prefix')
+
+        if load_envvars:
+            statsd_host = statsd_host or os.environ.get('STATSD_HOST')
+            statsd_port = statsd_port or os.environ.get('STATSD_PORT')
+            statsd_prefix = statsd_prefix or os.environ.get('STATSD_PREFIX')
+
+        self._statsd_host = statsd_host
+        self._statsd_port = statsd_port
+        self._statsd_prefix = statsd_prefix
 
         # Flag location to hold the peeked value of an argparse timeout value
         self._argv_timeout = False
@@ -323,6 +300,29 @@ class OpenStackConfig(object):
         # password = self._pw_callback(prompt="Password: ")
         self._pw_callback = pw_func
 
+    def _get_os_environ(self, envvar_prefix=None):
+        ret = self._defaults_module.get_defaults()
+        if not envvar_prefix:
+            # This makes the or below be OS_ or OS_ which is a no-op
+            envvar_prefix = 'OS_'
+        environkeys = [k for k in os.environ.keys()
+                       if (k.startswith('OS_') or k.startswith(envvar_prefix))
+                       and not k.startswith('OS_TEST')  # infra CI var
+                       and not k.startswith('OS_STD')   # oslotest var
+                       and not k.startswith('OS_LOG')   # oslotest var
+                       ]
+        for k in environkeys:
+            newkey = k.split('_', 1)[-1].lower()
+            ret[newkey] = os.environ[k]
+        # If the only environ keys are selectors or behavior modification,
+        # don't return anything
+        selectors = set([
+            'OS_CLOUD', 'OS_REGION_NAME',
+            'OS_CLIENT_CONFIG_FILE', 'OS_CLIENT_SECURE_FILE', 'OS_CLOUD_NAME'])
+        if set(environkeys) - selectors:
+            return ret
+        return None
+
     def _get_envvar(self, key, default=None):
         if not self._load_envvars:
             return default
@@ -335,12 +335,12 @@ class OpenStackConfig(object):
         :param dict defaults: (optional) default values to merge under the
                                          found config
         """
-        defaults = self._normalize_keys(defaults or {})
+        defaults = _util.normalize_keys(defaults or {})
         if not key:
             return defaults
-        return _merge_clouds(
+        return _util.merge_clouds(
             defaults,
-            self._normalize_keys(self.cloud_config.get(key, {})))
+            _util.normalize_keys(self.cloud_config.get(key, {})))
 
     def _load_config_file(self):
         return self._load_yaml_json_file(self._config_files)
@@ -361,43 +361,6 @@ class OpenStackConfig(object):
                         return path, yaml.safe_load(f)
         return (None, {})
 
-    def _normalize_keys(self, config):
-        new_config = {}
-        for key, value in config.items():
-            key = key.replace('-', '_')
-            if isinstance(value, dict):
-                new_config[key] = self._normalize_keys(value)
-            elif isinstance(value, bool):
-                new_config[key] = value
-            elif isinstance(value, int) and key != 'verbose_level':
-                new_config[key] = str(value)
-            elif isinstance(value, float):
-                new_config[key] = str(value)
-            else:
-                new_config[key] = value
-        return new_config
-
-    def get_cache_expiration_time(self):
-        return int(self._cache_expiration_time)
-
-    def get_cache_interval(self):
-        return self.get_cache_expiration_time()
-
-    def get_cache_max_age(self):
-        return self.get_cache_expiration_time()
-
-    def get_cache_path(self):
-        return self._cache_path
-
-    def get_cache_class(self):
-        return self._cache_class
-
-    def get_cache_arguments(self):
-        return copy.deepcopy(self._cache_arguments)
-
-    def get_cache_expiration(self):
-        return copy.deepcopy(self._cache_expiration)
-
     def _expand_region_name(self, region_name):
         return {'name': region_name, 'values': {}}
 
@@ -405,6 +368,13 @@ class OpenStackConfig(object):
         ret = []
         for region in regions:
             if isinstance(region, dict):
+                # i.e. must have name key, and only name,values keys
+                if 'name' not in region or \
+                   not {'name', 'values'} >= set(region):
+                    raise exceptions.ConfigException(
+                        'Invalid region entry at: %s' % region)
+                if 'values' not in region:
+                    region['values'] = {}
                 ret.append(copy.deepcopy(region))
             else:
                 ret.append(self._expand_region_name(region))
@@ -420,7 +390,7 @@ class OpenStackConfig(object):
         return regions
 
     def _get_known_regions(self, cloud):
-        config = self._normalize_keys(self.cloud_config['clouds'][cloud])
+        config = _util.normalize_keys(self.cloud_config['clouds'][cloud])
         if 'regions' in config:
             return self._expand_regions(config['regions'])
         elif 'region_name' in config:
@@ -472,7 +442,7 @@ class OpenStackConfig(object):
     def get_cloud_names(self):
         return self.cloud_config['clouds'].keys()
 
-    def _get_base_cloud_config(self, name):
+    def _get_base_cloud_config(self, name, profile=None):
         cloud = dict()
 
         # Only validate cloud name if one was given
@@ -482,6 +452,8 @@ class OpenStackConfig(object):
                     name=name))
 
         our_cloud = self.cloud_config['clouds'].get(name, dict())
+        if profile:
+            our_cloud['profile'] = profile
 
         # Get the defaults
         cloud.update(self.defaults)
@@ -500,35 +472,43 @@ class OpenStackConfig(object):
         # Expand a profile if it exists. 'cloud' is an old confusing name
         # for this.
         profile_name = our_cloud.get('profile', our_cloud.get('cloud', None))
-        if profile_name and profile_name != self.envvar_key:
-            if 'cloud' in our_cloud:
-                warnings.warn(
-                    "{0} use the keyword 'cloud' to reference a known "
-                    "vendor profile. This has been deprecated in favor of the "
-                    "'profile' keyword.".format(self.config_filename))
-            vendor_filename, vendor_file = self._load_vendor_file()
-            if vendor_file and profile_name in vendor_file['public-clouds']:
-                _auth_update(cloud, vendor_file['public-clouds'][profile_name])
+        if not profile_name or profile_name == self.envvar_key:
+            return
+        if 'cloud' in our_cloud:
+            warnings.warn(
+                "{0} uses the keyword 'cloud' to reference a known "
+                "vendor profile. This has been deprecated in favor of the "
+                "'profile' keyword.".format(self.config_filename))
+
+        vendor_filename, vendor_file = self._load_vendor_file()
+        if (vendor_file and 'public-clouds' in vendor_file
+                and profile_name in vendor_file['public-clouds']):
+            _auth_update(cloud, vendor_file['public-clouds'][profile_name])
+        else:
+            profile_data = vendors.get_profile(profile_name)
+            if profile_data:
+                nested_profile = profile_data.pop('profile', None)
+                if nested_profile:
+                    nested_profile_data = vendors.get_profile(nested_profile)
+                    if nested_profile_data:
+                        profile_data = nested_profile_data
+                status = profile_data.pop('status', 'active')
+                message = profile_data.pop('message', '')
+                if status == 'deprecated':
+                    warnings.warn(
+                        "{profile_name} is deprecated: {message}".format(
+                            profile_name=profile_name, message=message))
+                elif status == 'shutdown':
+                    raise exceptions.ConfigException(
+                        "{profile_name} references a cloud that no longer"
+                        " exists: {message}".format(
+                            profile_name=profile_name, message=message))
+                _auth_update(cloud, profile_data)
             else:
-                profile_data = vendors.get_profile(profile_name)
-                if profile_data:
-                    status = profile_data.pop('status', 'active')
-                    message = profile_data.pop('message', '')
-                    if status == 'deprecated':
-                        warnings.warn(
-                            "{profile_name} is deprecated: {message}".format(
-                                profile_name=profile_name, message=message))
-                    elif status == 'shutdown':
-                        raise exceptions.ConfigException(
-                            "{profile_name} references a cloud that no longer"
-                            " exists: {message}".format(
-                                profile_name=profile_name, message=message))
-                    _auth_update(cloud, profile_data)
-                else:
-                    # Can't find the requested vendor config, go about business
-                    warnings.warn("Couldn't find the vendor profile '{0}', for"
-                                  " the cloud '{1}'".format(profile_name,
-                                                            name))
+                # Can't find the requested vendor config, go about business
+                warnings.warn("Couldn't find the vendor profile '{0}', for"
+                              " the cloud '{1}'".format(profile_name,
+                                                        name))
 
     def _project_scoped(self, cloud):
         return ('project_id' in cloud or 'project_name' in cloud
@@ -882,7 +862,7 @@ class OpenStackConfig(object):
         return config
 
     def _get_auth_loader(self, config):
-        # Re-use the admin_token plugin for the "None" plugin
+        # Use the 'none' plugin for variants of None specified,
         # since it does not look up endpoints or tokens but rather
         # does a passthrough. This is useful for things like Ironic
         # that have a keystoneless operational mode, but means we're
@@ -971,9 +951,9 @@ class OpenStackConfig(object):
     def option_prompt(self, config, p_opt):
         """Prompt user for option that requires a value"""
         if (
-                getattr(p_opt, 'prompt', None) is not None and
-                p_opt.dest not in config['auth'] and
-                self._pw_callback is not None
+                getattr(p_opt, 'prompt', None) is not None
+                and p_opt.dest not in config['auth']
+                and self._pw_callback is not None
         ):
             config['auth'][p_opt.dest] = self._pw_callback(p_opt.prompt)
         return config
@@ -1000,9 +980,9 @@ class OpenStackConfig(object):
         """Perform the set of magic argument fixups"""
 
         # Infer token plugin if a token was given
-        if (('auth' in config and 'token' in config['auth']) or
-                ('auth_token' in config and config['auth_token']) or
-                ('token' in config and config['token'])):
+        if (('auth' in config and 'token' in config['auth'])
+                or ('auth_token' in config and config['auth_token'])
+                or ('token' in config and config['token'])):
             config.setdefault('token', config.pop('auth_token', None))
 
         # These backwards compat values are only set via argparse. If it's
@@ -1052,6 +1032,7 @@ class OpenStackConfig(object):
             on missing required auth parameters
         """
 
+        profile = kwargs.pop('profile', None)
         args = self._fix_args(kwargs, argparse=argparse)
 
         if cloud is None:
@@ -1060,7 +1041,7 @@ class OpenStackConfig(object):
             else:
                 cloud = self.default_cloud
 
-        config = self._get_base_cloud_config(cloud)
+        config = self._get_base_cloud_config(cloud, profile)
 
         # Get region specific settings
         if 'region_name' not in args:
@@ -1083,7 +1064,7 @@ class OpenStackConfig(object):
                         config[key] = val
 
         config = self.magic_fixes(config)
-        config = self._normalize_keys(config)
+        config = _util.normalize_keys(config)
 
         # NOTE(dtroyer): OSC needs a hook into the auth args before the
         #                plugin is loaded in order to maintain backward-
@@ -1111,16 +1092,26 @@ class OpenStackConfig(object):
             cloud_name = ''
         else:
             cloud_name = str(cloud)
-        return cloud_region.CloudRegion(
+        return self._cloud_region_class(
             name=cloud_name,
             region_name=config['region_name'],
             config=config,
+            extra_config=self.extra_config,
             force_ipv4=force_ipv4,
             auth_plugin=auth_plugin,
             openstack_config=self,
             session_constructor=self._session_constructor,
             app_name=self._app_name,
             app_version=self._app_version,
+            cache_expiration_time=self._cache_expiration_time,
+            cache_expirations=self._cache_expirations,
+            cache_path=self._cache_path,
+            cache_class=self._cache_class,
+            cache_arguments=self._cache_arguments,
+            password_callback=self._pw_callback,
+            statsd_host=self._statsd_host,
+            statsd_port=self._statsd_port,
+            statsd_prefix=self._statsd_prefix,
         )
     # TODO(mordred) Backwards compat for OSC transition
     get_one_cloud = get_one
@@ -1208,13 +1199,20 @@ class OpenStackConfig(object):
             cloud_name = ''
         else:
             cloud_name = str(cloud)
-        return cloud_region.CloudRegion(
+        return self._cloud_region_class(
             name=cloud_name,
             region_name=config['region_name'],
-            config=self._normalize_keys(config),
+            config=config,
+            extra_config=self.extra_config,
             force_ipv4=force_ipv4,
             auth_plugin=auth_plugin,
             openstack_config=self,
+            cache_expiration_time=self._cache_expiration_time,
+            cache_expirations=self._cache_expirations,
+            cache_path=self._cache_path,
+            cache_class=self._cache_class,
+            cache_arguments=self._cache_arguments,
+            password_callback=self._pw_callback,
         )
 
     @staticmethod
@@ -1248,6 +1246,7 @@ class OpenStackConfig(object):
         with open(config_file, 'w') as fh:
             yaml.safe_dump(cur_config, fh, default_flow_style=False)
 
+
 if __name__ == '__main__':
     config = OpenStackConfig().get_all_clouds()
     for cloud in config:
