@@ -13,15 +13,20 @@
 import mock
 
 from openstack.baremetal.v1 import _proxy
+from openstack.baremetal.v1 import allocation
 from openstack.baremetal.v1 import chassis
 from openstack.baremetal.v1 import driver
 from openstack.baremetal.v1 import node
 from openstack.baremetal.v1 import port
+from openstack.baremetal.v1 import port_group
 from openstack import exceptions
 from openstack.tests.unit import base
 from openstack.tests.unit import test_proxy_base
 
 
+_MOCK_METHOD = 'openstack.baremetal.v1._proxy.Proxy._get_with_fields'
+
+
 class TestBaremetalProxy(test_proxy_base.TestProxyBase):
 
     def setUp(self):
@@ -29,22 +34,22 @@ class TestBaremetalProxy(test_proxy_base.TestProxyBase):
         self.proxy = _proxy.Proxy(self.session)
 
     def test_drivers(self):
-        self.verify_list(self.proxy.drivers, driver.Driver, paginated=False)
+        self.verify_list(self.proxy.drivers, driver.Driver)
 
     def test_get_driver(self):
         self.verify_get(self.proxy.get_driver, driver.Driver)
 
-    def test_chassis_detailed(self):
-        self.verify_list(self.proxy.chassis, chassis.ChassisDetail,
-                         paginated=True,
-                         method_kwargs={"details": True, "query": 1},
-                         expected_kwargs={"query": 1})
+    @mock.patch.object(chassis.Chassis, 'list')
+    def test_chassis_detailed(self, mock_list):
+        result = self.proxy.chassis(details=True, query=1)
+        self.assertIs(result, mock_list.return_value)
+        mock_list.assert_called_once_with(self.proxy, details=True, query=1)
 
-    def test_chassis_not_detailed(self):
-        self.verify_list(self.proxy.chassis, chassis.Chassis,
-                         paginated=True,
-                         method_kwargs={"details": False, "query": 1},
-                         expected_kwargs={"query": 1})
+    @mock.patch.object(chassis.Chassis, 'list')
+    def test_chassis_not_detailed(self, mock_list):
+        result = self.proxy.chassis(query=1)
+        self.assertIs(result, mock_list.return_value)
+        mock_list.assert_called_once_with(self.proxy, details=False, query=1)
 
     def test_create_chassis(self):
         self.verify_create(self.proxy.create_chassis, chassis.Chassis)
@@ -53,7 +58,9 @@ class TestBaremetalProxy(test_proxy_base.TestProxyBase):
         self.verify_find(self.proxy.find_chassis, chassis.Chassis)
 
     def test_get_chassis(self):
-        self.verify_get(self.proxy.get_chassis, chassis.Chassis)
+        self.verify_get(self.proxy.get_chassis, chassis.Chassis,
+                        mock_method=_MOCK_METHOD,
+                        expected_kwargs={'fields': None})
 
     def test_update_chassis(self):
         self.verify_update(self.proxy.update_chassis, chassis.Chassis)
@@ -64,17 +71,17 @@ class TestBaremetalProxy(test_proxy_base.TestProxyBase):
     def test_delete_chassis_ignore(self):
         self.verify_delete(self.proxy.delete_chassis, chassis.Chassis, True)
 
-    def test_nodes_detailed(self):
-        self.verify_list(self.proxy.nodes, node.NodeDetail,
-                         paginated=True,
-                         method_kwargs={"details": True, "query": 1},
-                         expected_kwargs={"query": 1})
+    @mock.patch.object(node.Node, 'list')
+    def test_nodes_detailed(self, mock_list):
+        result = self.proxy.nodes(details=True, query=1)
+        self.assertIs(result, mock_list.return_value)
+        mock_list.assert_called_once_with(self.proxy, details=True, query=1)
 
-    def test_nodes_not_detailed(self):
-        self.verify_list(self.proxy.nodes, node.Node,
-                         paginated=True,
-                         method_kwargs={"details": False, "query": 1},
-                         expected_kwargs={"query": 1})
+    @mock.patch.object(node.Node, 'list')
+    def test_nodes_not_detailed(self, mock_list):
+        result = self.proxy.nodes(query=1)
+        self.assertIs(result, mock_list.return_value)
+        mock_list.assert_called_once_with(self.proxy, details=False, query=1)
 
     def test_create_node(self):
         self.verify_create(self.proxy.create_node, node.Node)
@@ -83,7 +90,9 @@ class TestBaremetalProxy(test_proxy_base.TestProxyBase):
         self.verify_find(self.proxy.find_node, node.Node)
 
     def test_get_node(self):
-        self.verify_get(self.proxy.get_node, node.Node)
+        self.verify_get(self.proxy.get_node, node.Node,
+                        mock_method=_MOCK_METHOD,
+                        expected_kwargs={'fields': None})
 
     @mock.patch.object(node.Node, 'commit', autospec=True)
     def test_update_node(self, mock_commit):
@@ -106,17 +115,17 @@ class TestBaremetalProxy(test_proxy_base.TestProxyBase):
     def test_delete_node_ignore(self):
         self.verify_delete(self.proxy.delete_node, node.Node, True)
 
-    def test_ports_detailed(self):
-        self.verify_list(self.proxy.ports, port.PortDetail,
-                         paginated=True,
-                         method_kwargs={"details": True, "query": 1},
-                         expected_kwargs={"query": 1})
+    @mock.patch.object(port.Port, 'list')
+    def test_ports_detailed(self, mock_list):
+        result = self.proxy.ports(details=True, query=1)
+        self.assertIs(result, mock_list.return_value)
+        mock_list.assert_called_once_with(self.proxy, details=True, query=1)
 
-    def test_ports_not_detailed(self):
-        self.verify_list(self.proxy.ports, port.Port,
-                         paginated=True,
-                         method_kwargs={"details": False, "query": 1},
-                         expected_kwargs={"query": 1})
+    @mock.patch.object(port.Port, 'list')
+    def test_ports_not_detailed(self, mock_list):
+        result = self.proxy.ports(query=1)
+        self.assertIs(result, mock_list.return_value)
+        mock_list.assert_called_once_with(self.proxy, details=False, query=1)
 
     def test_create_port(self):
         self.verify_create(self.proxy.create_port, port.Port)
@@ -125,7 +134,9 @@ class TestBaremetalProxy(test_proxy_base.TestProxyBase):
         self.verify_find(self.proxy.find_port, port.Port)
 
     def test_get_port(self):
-        self.verify_get(self.proxy.get_port, port.Port)
+        self.verify_get(self.proxy.get_port, port.Port,
+                        mock_method=_MOCK_METHOD,
+                        expected_kwargs={'fields': None})
 
     def test_update_port(self):
         self.verify_update(self.proxy.update_port, port.Port)
@@ -136,6 +147,55 @@ class TestBaremetalProxy(test_proxy_base.TestProxyBase):
     def test_delete_port_ignore(self):
         self.verify_delete(self.proxy.delete_port, port.Port, True)
 
+    @mock.patch.object(port_group.PortGroup, 'list')
+    def test_port_groups_detailed(self, mock_list):
+        result = self.proxy.port_groups(details=True, query=1)
+        self.assertIs(result, mock_list.return_value)
+        mock_list.assert_called_once_with(self.proxy, details=True, query=1)
+
+    @mock.patch.object(port_group.PortGroup, 'list')
+    def test_port_groups_not_detailed(self, mock_list):
+        result = self.proxy.port_groups(query=1)
+        self.assertIs(result, mock_list.return_value)
+        mock_list.assert_called_once_with(self.proxy, details=False, query=1)
+
+    def test_get_port_group(self):
+        self.verify_get(self.proxy.get_port_group, port_group.PortGroup,
+                        mock_method=_MOCK_METHOD,
+                        expected_kwargs={'fields': None})
+
+    def test_create_allocation(self):
+        self.verify_create(self.proxy.create_allocation, allocation.Allocation)
+
+    def test_get_allocation(self):
+        self.verify_get(self.proxy.get_allocation, allocation.Allocation,
+                        mock_method=_MOCK_METHOD,
+                        expected_kwargs={'fields': None})
+
+    def test_delete_allocation(self):
+        self.verify_delete(self.proxy.delete_allocation, allocation.Allocation,
+                           False)
+
+    def test_delete_allocation_ignore(self):
+        self.verify_delete(self.proxy.delete_allocation, allocation.Allocation,
+                           True)
+
+    @mock.patch.object(node.Node, 'fetch', autospec=True)
+    def test__get_with_fields_none(self, mock_fetch):
+        result = self.proxy._get_with_fields(node.Node, 'value')
+        self.assertIs(result, mock_fetch.return_value)
+        mock_fetch.assert_called_once_with(mock.ANY, self.proxy,
+                                           error_message=mock.ANY)
+
+    @mock.patch.object(node.Node, 'fetch', autospec=True)
+    def test__get_with_fields(self, mock_fetch):
+        result = self.proxy._get_with_fields(node.Node, 'value',
+                                             fields=['a', 'b'])
+        self.assertIs(result, mock_fetch.return_value)
+        mock_fetch.assert_called_once_with(mock.ANY, self.proxy,
+                                           error_message=mock.ANY,
+                                           fields='a,b')
+
 
 @mock.patch('time.sleep', lambda _sec: None)
 @mock.patch.object(_proxy.Proxy, 'get_node', autospec=True)
