@@ -11,22 +11,19 @@
 # WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 # License for the specific language governing permissions and limitations
 # under the License.
-
 import operator
 import tempfile
 import uuid
 
 import six
 
+from openstack import exceptions
 from openstack.cloud import exc
 from openstack.cloud import meta
 from openstack.tests import fakes
 from openstack.tests.unit import base
 
 
-CINDER_URL = 'https://volume.example.com/v2/1c36b64c840a42cd9e9b931a369337f0'
-
-
 class BaseTestImage(base.TestCase):
 
     def setUp(self):
@@ -35,12 +32,15 @@ class BaseTestImage(base.TestCase):
         self.image_name = self.getUniqueString('image')
         self.object_name = u'images/{name}'.format(name=self.image_name)
         self.imagefile = tempfile.NamedTemporaryFile(delete=False)
-        self.imagefile.write(b'\0')
+        data = b'\2\0'
+        self.imagefile.write(data)
         self.imagefile.close()
+        self.output = data
         self.fake_image_dict = fakes.make_fake_image(
-            image_id=self.image_id, image_name=self.image_name)
+            image_id=self.image_id, image_name=self.image_name,
+            data=self.imagefile.name
+        )
         self.fake_search_return = {'images': [self.fake_image_dict]}
-        self.output = uuid.uuid4().bytes
         self.container_name = self.getUniqueString('container')
 
 
@@ -332,9 +332,13 @@ class TestImage(BaseTestImage):
                          u'container_format': u'bare',
                          u'disk_format': u'qcow2',
                          u'name': self.image_name,
-                         u'owner_specified.openstack.md5': fakes.NO_MD5,
+                         u'owner_specified.openstack.md5':
+                             self.fake_image_dict[
+                                 'owner_specified.openstack.md5'],
                          u'owner_specified.openstack.object': self.object_name,
-                         u'owner_specified.openstack.sha256': fakes.NO_SHA256,
+                         u'owner_specified.openstack.sha256':
+                             self.fake_image_dict[
+                                 'owner_specified.openstack.sha256'],
                          u'visibility': u'private'})
                  ),
             dict(method='PUT',
@@ -359,7 +363,8 @@ class TestImage(BaseTestImage):
             is_public=False)
 
         self.assert_calls()
-        self.assertEqual(self.adapter.request_history[5].text.read(), b'\x00')
+        self.assertEqual(self.adapter.request_history[5].text.read(),
+                         self.output)
 
     def test_create_image_task(self):
         self.cloud.image_api_use_tasks = True
@@ -427,8 +432,12 @@ class TestImage(BaseTestImage):
                      object=self.image_name),
                  status_code=201,
                  validate=dict(
-                     headers={'x-object-meta-x-sdk-md5': fakes.NO_MD5,
-                              'x-object-meta-x-sdk-sha256': fakes.NO_SHA256})
+                     headers={'x-object-meta-x-sdk-md5':
+                              self.fake_image_dict[
+                                  'owner_specified.openstack.md5'],
+                              'x-object-meta-x-sdk-sha256':
+                              self.fake_image_dict[
+                                  'owner_specified.openstack.sha256']})
                  ),
             dict(method='POST',
                  uri=self.get_mock_url(
@@ -466,9 +475,13 @@ class TestImage(BaseTestImage):
                               container=self.container_name,
                               object=self.image_name),
                           u'path': u'/owner_specified.openstack.object'},
-                         {u'op': u'add', u'value': fakes.NO_MD5,
+                         {u'op': u'add', u'value':
+                             self.fake_image_dict[
+                                 'owner_specified.openstack.md5'],
                           u'path': u'/owner_specified.openstack.md5'},
-                         {u'op': u'add', u'value': fakes.NO_SHA256,
+                         {u'op': u'add', u'value':
+                             self.fake_image_dict[
+                                 'owner_specified.openstack.sha256'],
                           u'path': u'/owner_specified.openstack.sha256'}],
                          key=operator.itemgetter('path')),
                      headers={
@@ -485,8 +498,12 @@ class TestImage(BaseTestImage):
                      'X-Trans-Id': 'txbbb825960a3243b49a36f-005a0dadaedfw1',
                      'Content-Length': '1290170880',
                      'Last-Modified': 'Tue, 14 Apr 2015 18:29:01 GMT',
-                     'X-Object-Meta-X-Sdk-Sha256': fakes.NO_SHA256,
-                     'X-Object-Meta-X-Sdk-Md5': fakes.NO_MD5,
+                     'X-Object-Meta-X-Sdk-Sha256':
+                     self.fake_image_dict[
+                         'owner_specified.openstack.sha256'],
+                     'X-Object-Meta-X-Sdk-Md5':
+                     self.fake_image_dict[
+                         'owner_specified.openstack.md5'],
                      'Date': 'Thu, 16 Nov 2017 15:24:30 GMT',
                      'Accept-Ranges': 'bytes',
                      'Content-Type': 'application/octet-stream',
@@ -516,6 +533,54 @@ class TestImage(BaseTestImage):
         self.assertFalse(deleted)
         self.assert_calls()
 
+    def test_delete_image_task(self):
+        self.cloud.image_api_use_tasks = True
+        endpoint = self.cloud._object_store_client.get_endpoint()
+
+        object_path = self.fake_image_dict['owner_specified.openstack.object']
+
+        image_no_checksums = self.fake_image_dict.copy()
+        del(image_no_checksums['owner_specified.openstack.md5'])
+        del(image_no_checksums['owner_specified.openstack.sha256'])
+        del(image_no_checksums['owner_specified.openstack.object'])
+
+        self.register_uris([
+            dict(method='GET',
+                 uri=self.get_mock_url(
+                     'image', append=['images'], base_url_append='v2'),
+                 json=self.fake_search_return),
+            dict(method='DELETE',
+                 uri='https://image.example.com/v2/images/{id}'.format(
+                     id=self.image_id)),
+            dict(method='HEAD',
+                 uri='{endpoint}/{object}'.format(
+                     endpoint=endpoint,
+                     object=object_path),
+                 headers={
+                     'X-Timestamp': '1429036140.50253',
+                     'X-Trans-Id': 'txbbb825960a3243b49a36f-005a0dadaedfw1',
+                     'Content-Length': '1290170880',
+                     'Last-Modified': 'Tue, 14 Apr 2015 18:29:01 GMT',
+                     'X-Object-Meta-X-Sdk-Sha256':
+                     self.fake_image_dict[
+                         'owner_specified.openstack.sha256'],
+                     'X-Object-Meta-X-Sdk-Md5':
+                     self.fake_image_dict[
+                         'owner_specified.openstack.md5'],
+                     'Date': 'Thu, 16 Nov 2017 15:24:30 GMT',
+                     'Accept-Ranges': 'bytes',
+                     'Content-Type': 'application/octet-stream',
+                     'Etag': fakes.NO_MD5}),
+            dict(method='DELETE',
+                 uri='{endpoint}/{object}'.format(
+                     endpoint=endpoint,
+                     object=object_path)),
+        ])
+
+        self.cloud.delete_image(self.image_id)
+
+        self.assert_calls()
+
     def test_delete_autocreated_image_objects(self):
         self.use_keystone_v3()
         self.cloud.image_api_use_tasks = True
@@ -755,46 +820,56 @@ class TestImage(BaseTestImage):
     def test_create_image_put_v2_wrong_checksum_delete(self):
         self.cloud.image_api_use_tasks = False
 
-        args = {'name': self.image_name,
-                'container_format': 'bare', 'disk_format': 'qcow2',
-                'owner_specified.openstack.md5': fakes.NO_MD5,
-                'owner_specified.openstack.sha256': fakes.NO_SHA256,
-                'owner_specified.openstack.object': 'images/{name}'.format(
-                    name=self.image_name),
-                'visibility': 'private'}
+        fake_image = self.fake_image_dict
 
-        ret = args.copy()
-        ret['id'] = self.image_id
-        ret['status'] = 'success'
-        ret['checksum'] = 'fake'
+        fake_image['owner_specified.openstack.md5'] = 'a'
+        fake_image['owner_specified.openstack.sha256'] = 'b'
 
         self.register_uris([
             dict(method='GET',
-                 uri='https://image.example.com/v2/images',
+                 uri=self.get_mock_url(
+                     'image', append=['images'], base_url_append='v2'),
                  json={'images': []}),
             dict(method='POST',
-                 uri='https://image.example.com/v2/images',
-                 json=ret,
-                 validate=dict(json=args)),
-            dict(method='PUT',
-                 uri='https://image.example.com/v2/images/{id}/file'.format(
-                     id=self.image_id),
-                 status_code=400,
+                 uri=self.get_mock_url(
+                     'image', append=['images'], base_url_append='v2'),
+                 json=self.fake_image_dict,
                  validate=dict(
-                     headers={
-                         'Content-Type': 'application/octet-stream',
-                     },
-                 )),
+                     json={
+                         u'container_format': u'bare',
+                         u'disk_format': u'qcow2',
+                         u'name': self.image_name,
+                         u'owner_specified.openstack.md5':
+                             fake_image[
+                                 'owner_specified.openstack.md5'],
+                         u'owner_specified.openstack.object': self.object_name,
+                         u'owner_specified.openstack.sha256':
+                             fake_image[
+                                 'owner_specified.openstack.sha256'],
+                         u'visibility': u'private'})
+                 ),
+            dict(method='PUT',
+                 uri=self.get_mock_url(
+                     'image', append=['images', self.image_id, 'file'],
+                     base_url_append='v2'),
+                 request_headers={'Content-Type': 'application/octet-stream'}),
+            dict(method='GET',
+                 uri=self.get_mock_url(
+                     'image', append=['images', self.fake_image_dict['id']],
+                     base_url_append='v2'
+                 ),
+                 json=fake_image),
             dict(method='DELETE',
                  uri='https://image.example.com/v2/images/{id}'.format(
-                     id=self.image_id)),
+                     id=self.image_id))
         ])
 
         self.assertRaises(
-            exc.OpenStackCloudHTTPError,
-            self._call_create_image,
-            self.image_name,
-            md5='some_fake')
+            exceptions.SDKException,
+            self.cloud.create_image,
+            self.image_name, self.imagefile.name,
+            is_public=False, md5='a', sha256='b'
+        )
 
         self.assert_calls()
 
@@ -967,7 +1042,9 @@ class TestImageSuburl(BaseTestImage):
 
     def setUp(self):
         super(TestImageSuburl, self).setUp()
-        self.use_keystone_v3(catalog='catalog-v3-suburl.json')
+        self.os_fixture.use_suburl()
+        self.os_fixture.build_tokens()
+        self.use_keystone_v3()
         self.use_glance(
             image_version_json='image-version-suburl.json',
             image_discovery_url='https://example.com/image')
@@ -1064,7 +1141,6 @@ class TestImageVolume(BaseTestImage):
         self.volume_id = str(uuid.uuid4())
 
     def test_create_image_volume(self):
-
         self.register_uris([
             self.get_cinder_discovery_mock_dict(),
             dict(method='POST',
