@@ -30,10 +30,9 @@ class TestRole(base.TestCase):
         self.assertEqual('role', sot.resource_key)
         self.assertEqual('roles', sot.resources_key)
         self.assertEqual('/OS-KSADM/roles', sot.base_path)
-        self.assertEqual('identity', sot.service.service_type)
         self.assertTrue(sot.allow_create)
-        self.assertTrue(sot.allow_get)
-        self.assertTrue(sot.allow_update)
+        self.assertTrue(sot.allow_fetch)
+        self.assertTrue(sot.allow_commit)
         self.assertTrue(sot.allow_delete)
         self.assertTrue(sot.allow_list)
 
