@@ -42,10 +42,9 @@ class TestSubnetpool(base.TestCase):
         self.assertEqual('subnetpool', sot.resource_key)
         self.assertEqual('subnetpools', sot.resources_key)
         self.assertEqual('/subnetpools', sot.base_path)
-        self.assertEqual('network', sot.service.service_type)
         self.assertTrue(sot.allow_create)
-        self.assertTrue(sot.allow_get)
-        self.assertTrue(sot.allow_update)
+        self.assertTrue(sot.allow_fetch)
+        self.assertTrue(sot.allow_commit)
         self.assertTrue(sot.allow_delete)
         self.assertTrue(sot.allow_list)
 
